@@ -18,7 +18,7 @@ use crate::services::{
     protocol_type_checker::ProtocolTypeCheckerService,
 };
 use crate::{Behaviour, CKBProtocol, Peer, PeerIndex, ProtocolId, ServiceControl};
-use ckb_app_config::NetworkConfig;
+use ckb_app_config::{NetworkConfig, SupportProtocol};
 use ckb_logger::{debug, error, info, trace, warn};
 use ckb_spawn::Spawn;
 use ckb_stop_handler::{SignalSender, StopHandler};
@@ -75,12 +75,14 @@ pub struct NetworkState {
     pending_observed_addrs: RwLock<HashSet<Multiaddr>>,
     local_private_key: secio::SecioKeyPair,
     local_peer_id: PeerId,
-    bootnodes: Vec<Multiaddr>,
+    pub(crate) bootnodes: Vec<Multiaddr>,
     pub(crate) config: NetworkConfig,
     pub(crate) active: AtomicBool,
     /// Node supported protocols
     /// fields: ProtocolId, Protocol Name, Supported Versions
     pub(crate) protocols: RwLock<Vec<(ProtocolId, String, Vec<String>)>>,
+
+    pub(crate) ckb2021: AtomicBool,
 }
 
 impl NetworkState {
@@ -131,9 +133,16 @@ impl NetworkState {
             local_peer_id,
             active: AtomicBool::new(true),
             protocols: RwLock::new(Vec::new()),
+            ckb2021: AtomicBool::new(false),
         })
     }
 
+    /// fork flag
+    pub fn ckb2021(self, init: bool) -> Self {
+        self.ckb2021.store(init, Ordering::SeqCst);
+        self
+    }
+
     pub(crate) fn report_session(
         &self,
         p2p_control: &ServiceControl,
@@ -146,16 +155,7 @@ impl NetworkState {
                 .map(|peer| peer.connected_addr.clone())
         }) {
             trace!("report {:?} because {:?}", addr, behaviour);
-            let report_result = match self.peer_store.lock().report(&addr, behaviour) {
-                Ok(result) => result,
-                Err(err) => {
-                    error!(
-                        "Report failed addr: {:?} behaviour: {:?} error: {:?}",
-                        addr, behaviour, err
-                    );
-                    return;
-                }
-            };
+            let report_result = self.peer_store.lock().report(&addr, behaviour);
             if report_result.is_banned() {
                 if let Err(err) = disconnect_with_message(p2p_control, session_id, "banned") {
                     debug!("Disconnect failed {:?}, error: {:?}", session_id, err);
@@ -226,8 +226,8 @@ impl NetworkState {
         accept_peer_result.map_err(Into::into)
     }
 
-    // For restrict lock in inner scope
-    pub(crate) fn with_peer_registry<F, T>(&self, callback: F) -> T
+    /// For restrict lock in inner scope
+    pub fn with_peer_registry<F, T>(&self, callback: F) -> T
     where
         F: FnOnce(&PeerRegistry) -> T,
     {
@@ -256,8 +256,7 @@ impl NetworkState {
     }
 
     /// Use on test
-    #[allow(dead_code)]
-    pub(crate) fn local_private_key(&self) -> &secio::SecioKeyPair {
+    pub fn local_private_key(&self) -> &secio::SecioKeyPair {
         &self.local_private_key
     }
 
@@ -479,6 +478,16 @@ pub struct EventHandler<T> {
     pub(crate) exit_handler: T,
 }
 
+impl<T> EventHandler<T> {
+    /// init an event handler
+    pub fn new(network_state: Arc<NetworkState>, exit_handler: T) -> Self {
+        Self {
+            network_state,
+            exit_handler,
+        }
+    }
+}
+
 /// Exit trait used to notify all other module to exit
 pub trait ExitHandler: Send + Unpin + 'static {
     /// notify other module to exit
@@ -661,7 +670,7 @@ impl<T: ExitHandler> ServiceHandle for EventHandler<T> {
                     match self.network_state.accept_peer(&session_context) {
                         Ok(Some(evicted_peer)) => {
                             debug!(
-                                "evict peer (disonnect it), {} => {}",
+                                "evict peer (disconnect it), {} => {}",
                                 evicted_peer.session_id, evicted_peer.connected_addr,
                             );
                             if let Err(err) = disconnect_with_message(
@@ -731,7 +740,7 @@ impl<T: ExitHandler> ServiceHandle for EventHandler<T> {
 pub struct NetworkService<T> {
     p2p_service: Service<EventHandler<T>>,
     network_state: Arc<NetworkState>,
-    ping_controller: Sender<()>,
+    ping_controller: Option<Sender<()>>,
     // Background services
     bg_services: Vec<Pin<Box<dyn Future<Output = ()> + 'static + Send>>>,
     version: String,
@@ -748,66 +757,83 @@ impl<T: ExitHandler> NetworkService<T> {
         exit_handler: T,
     ) -> Self {
         let config = &network_state.config;
-        // == Build special protocols
-
-        // TODO: how to deny banned node to open those protocols?
-        // Ping protocol
-        let ping_interval = Duration::from_secs(config.ping_interval_secs);
-        let ping_timeout = Duration::from_secs(config.ping_timeout_secs);
-
-        let ping_network_state = Arc::clone(&network_state);
-        let (ping_handler, ping_controller) =
-            PingHandler::new(ping_interval, ping_timeout, ping_network_state);
-        let ping_meta = SupportProtocols::Ping.build_meta_with_service_handle(move || {
-            ProtocolHandle::Callback(Box::new(ping_handler))
-        });
-
-        // Discovery protocol
-        let addr_mgr = DiscoveryAddressManager {
-            network_state: Arc::clone(&network_state),
-            discovery_local_address: config.discovery_local_address,
-        };
-        let disc_meta = SupportProtocols::Discovery.build_meta_with_service_handle(move || {
-            ProtocolHandle::Callback(Box::new(DiscoveryProtocol::new(
-                addr_mgr,
-                config
-                    .discovery_announce_check_interval_secs
-                    .map(Duration::from_secs),
-            )))
-        });
-
-        // Identify protocol
-        let identify_callback =
-            IdentifyCallback::new(Arc::clone(&network_state), name, version.clone());
-        let identify_meta = SupportProtocols::Identify.build_meta_with_service_handle(move || {
-            ProtocolHandle::Callback(Box::new(IdentifyProtocol::new(identify_callback)))
-        });
-
-        // Feeler protocol
-        let feeler_meta = SupportProtocols::Feeler.build_meta_with_service_handle({
-            let network_state = Arc::clone(&network_state);
-            move || ProtocolHandle::Callback(Box::new(Feeler::new(Arc::clone(&network_state))))
-        });
-
-        let disconnect_message_state = Arc::clone(&network_state);
-        let disconnect_message_meta = SupportProtocols::DisconnectMessage
-            .build_meta_with_service_handle(move || {
-                ProtocolHandle::Callback(Box::new(DisconnectMessageProtocol::new(
-                    disconnect_message_state,
-                )))
-            });
-
         // == Build p2p service struct
         let mut protocol_metas = protocols
             .into_iter()
             .map(CKBProtocol::build)
             .collect::<Vec<_>>();
-        protocol_metas.push(feeler_meta);
-        protocol_metas.push(disconnect_message_meta);
-        protocol_metas.push(ping_meta);
-        protocol_metas.push(disc_meta);
+
+        // == Build special protocols
+
+        // Identify is a core protocol, user cannot disable it via config
+        let identify_callback =
+            IdentifyCallback::new(Arc::clone(&network_state), name, version.clone());
+        let identify_meta = SupportProtocols::Identify.build_meta_with_service_handle(move || {
+            ProtocolHandle::Callback(Box::new(IdentifyProtocol::new(identify_callback)))
+        });
         protocol_metas.push(identify_meta);
 
+        // Ping protocol
+        let ping_controller = if config.support_protocols.contains(&SupportProtocol::Ping) {
+            let ping_interval = Duration::from_secs(config.ping_interval_secs);
+            let ping_timeout = Duration::from_secs(config.ping_timeout_secs);
+
+            let ping_network_state = Arc::clone(&network_state);
+            let (ping_handler, ping_controller) =
+                PingHandler::new(ping_interval, ping_timeout, ping_network_state);
+            let ping_meta = SupportProtocols::Ping.build_meta_with_service_handle(move || {
+                ProtocolHandle::Callback(Box::new(ping_handler))
+            });
+            protocol_metas.push(ping_meta);
+            Some(ping_controller)
+        } else {
+            None
+        };
+
+        // Discovery protocol
+        if config
+            .support_protocols
+            .contains(&SupportProtocol::Discovery)
+        {
+            let addr_mgr = DiscoveryAddressManager {
+                network_state: Arc::clone(&network_state),
+                discovery_local_address: config.discovery_local_address,
+            };
+            let disc_meta = SupportProtocols::Discovery.build_meta_with_service_handle(move || {
+                ProtocolHandle::Callback(Box::new(DiscoveryProtocol::new(
+                    addr_mgr,
+                    config
+                        .discovery_announce_check_interval_secs
+                        .map(Duration::from_secs),
+                )))
+            });
+            protocol_metas.push(disc_meta);
+        }
+
+        // Feeler protocol
+        if config.support_protocols.contains(&SupportProtocol::Feeler) {
+            let feeler_meta = SupportProtocols::Feeler.build_meta_with_service_handle({
+                let network_state = Arc::clone(&network_state);
+                move || ProtocolHandle::Callback(Box::new(Feeler::new(Arc::clone(&network_state))))
+            });
+            protocol_metas.push(feeler_meta);
+        }
+
+        // DisconnectMessage protocol
+        if config
+            .support_protocols
+            .contains(&SupportProtocol::DisconnectMessage)
+        {
+            let disconnect_message_state = Arc::clone(&network_state);
+            let disconnect_message_meta = SupportProtocols::DisconnectMessage
+                .build_meta_with_service_handle(move || {
+                    ProtocolHandle::Callback(Box::new(DisconnectMessageProtocol::new(
+                        disconnect_message_state,
+                    )))
+                });
+            protocol_metas.push(disconnect_message_meta);
+        }
+
         let mut service_builder = ServiceBuilder::default();
         let yamux_config = YamuxConfig {
             max_stream_count: protocol_metas.len(),
@@ -830,11 +856,12 @@ impl<T: ExitHandler> NetworkService<T> {
             .yamux_config(yamux_config)
             .forever(true)
             .max_connection_number(1024)
-            .set_send_buffer_size(config.max_send_buffer());
+            .set_send_buffer_size(config.max_send_buffer())
+            .timeout(Duration::from_secs(5));
 
         #[cfg(target_os = "linux")]
         let p2p_service = {
-            if config.reuse {
+            if config.reuse_port_on_linux {
                 let iter = config.listen_addresses.iter();
 
                 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
@@ -970,13 +997,15 @@ impl<T: ExitHandler> NetworkService<T> {
                 .into_iter()
                 .map(|paddr| paddr.addr)
                 .collect();
-            addrs.extend(
-                self.network_state
-                    .bootnodes
-                    .iter()
-                    .take(count.saturating_sub(addrs.len()))
-                    .cloned(),
-            );
+            // Get bootnodes randomly
+            let bootnodes = self
+                .network_state
+                .bootnodes
+                .iter()
+                .choose_multiple(&mut rand::thread_rng(), count.saturating_sub(addrs.len()))
+                .into_iter()
+                .cloned();
+            addrs.extend(bootnodes);
             addrs
         });
 
@@ -1087,13 +1116,13 @@ impl<T: ExitHandler> NetworkService<T> {
             return Err(e);
         }
 
-        let stop = StopHandler::new(SignalSender::Tokio(sender), None);
+        let stop = StopHandler::new(SignalSender::Tokio(sender), None, "network".to_string());
         Ok(NetworkController {
             version,
             network_state,
             p2p_control,
             ping_controller,
-            stop,
+            stop: Some(stop),
         })
     }
 }
@@ -1104,11 +1133,21 @@ pub struct NetworkController {
     version: String,
     network_state: Arc<NetworkState>,
     p2p_control: ServiceControl,
-    ping_controller: Sender<()>,
-    stop: StopHandler<()>,
+    ping_controller: Option<Sender<()>>,
+    stop: Option<StopHandler<()>>,
 }
 
 impl NetworkController {
+    /// set ckb2021 start
+    pub fn init_ckb2021(&self) {
+        self.network_state.ckb2021.store(true, Ordering::SeqCst);
+    }
+
+    /// get ckb2021 flag
+    pub fn load_ckb2021(&self) -> bool {
+        self.network_state.ckb2021.load(Ordering::SeqCst)
+    }
+
     /// Node listen address list
     pub fn public_urls(&self, max_urls: usize) -> Vec<(String, u8)> {
         self.network_state.public_urls(max_urls)
@@ -1280,14 +1319,29 @@ impl NetworkController {
 
     /// Try ping all connected peers
     pub fn ping_peers(&self) {
-        let mut ping_controller = self.ping_controller.clone();
-        let _ignore = ping_controller.try_send(());
+        if let Some(mut ping_controller) = self.ping_controller.clone() {
+            let _ignore = ping_controller.try_send(());
+        }
+    }
+
+    /// Since a non-owning reference does not count towards ownership,
+    /// it will not prevent the value stored in the allocation from being dropped
+    pub fn non_owning_clone(&self) -> Self {
+        NetworkController {
+            stop: None,
+            version: self.version.clone(),
+            network_state: Arc::clone(&self.network_state),
+            p2p_control: self.p2p_control.clone(),
+            ping_controller: self.ping_controller.clone(),
+        }
     }
 }
 
 impl Drop for NetworkController {
     fn drop(&mut self) {
-        self.stop.try_send();
+        if let Some(ref mut stop) = self.stop {
+            stop.try_send(());
+        }
     }
 }
 
